function [varargout] = plotGSIPFeasibleSet(problem,indicesFreePars,valuesFixedPars,lb,ub,nrPointsInDir,varargin)
% <problemname>.plotGSIPFeasibleSet(IndFree,ValFixed,lb,ub,nrPointsInDir) 
% implicitly plots the feasible set in up to 3 dimension. Where IndFree are
% the indices of the xi that can vary, ValFixed are the values of all other
% xi, lb are the lower bounds of the variable xi and ub their upper bounds.
% The last input describes the number of evaluated points in every
% dimension.



%%check wether the given parameters have the correct size
%check the problem p
assert(isa(problem,'ImplicitGSIPProblem'),'This function is only implemented for an implicit GSIP');

%check indicesFreeParameters
if size(indicesFreePars,1)>1
    indicesFreePars=indicesFreePars';
end
assert(size(indicesFreePars,1)==1,'indicesFreeParameters must be a vector');
for i=1:size(indicesFreePars,2)
    assert(~mod(indicesFreePars(i),1),'please insert indices for ''indicesFreeParameters''');
    assert(indicesFreePars(i)>0&& indicesFreePars(i)<=problem.getNrUppLevVars,'At least one entry of ''indicesFreeParameters'' is not a valid index for a parameter')
end

%check values Fixed Parameters
if size(valuesFixedPars,1)>1
    valuesFixedPars=valuesFixedPars';
end
if size(valuesFixedPars,2)==0
    valuesFixedPars=zeros(1,0);
end
assert(size(valuesFixedPars,1)==1,'''valuesFixedParameters'' must be a vector');
assert((size(valuesFixedPars,2)+size(indicesFreePars,2))==problem.getNrUppLevVars,'''valuesFixedParameters'' must be a vector');


%check the bounds
if size(lb,1)>1
    lb=lb';
end
assert(size(lb,1)==1,'''lowerBounds'' must be a vector')
assert(size(lb,2)==size(indicesFreePars,2),'The dimension of ''lowerBounds'' must be the same as the number of free parameters')
assert(min(lb~=-inf)==1,'The lower bounds can not equal to -inf')

if size(ub,1)>1
    ub=lupperowerBounds';
end
assert(size(ub,1)==1,'''upperBounds'' must be a vector')
assert(size(lb,2)==size(indicesFreePars,2),'The dimension of ''upperBounds'' must be the same as the number of free parameters')
assert(min(lb~=inf)==1,'The upper bounds can not equal to inf')



%check nrPointsInDir
assert(size(nrPointsInDir,1)==1,'''nrPointsInDir'' must be an integer');
assert(size(nrPointsInDir,2)==1,'''nrPointsInDir'' must be an integer');
assert(~mod(nrPointsInDir,1),'please insert an integer for ''nrPointsInDir''');
assert(nrPointsInDir>0,'''nrPointsInDir'' must be positive''');


%check in which dimension the plot should be done
n=size(indicesFreePars,2);
assert(n==1||n==2||n==3,'we can only plot problems with one, two or three free parameters')
switch n
    case 1
        [h]=plotFeasibleSetOfGSIP1D(problem,indicesFreePars,valuesFixedPars,lb,ub,nrPointsInDir);
    case 2
        if nargin>6
            if strcmp(varargin{1},'heights')
                [h]=plotFeasibleSetOfGSIP2DHeightProfile(problem,indicesFreePars,valuesFixedPars,lb,ub,nrPointsInDir);
            else
                warning('To get a picture with boundare give as a last input ''heights''');
            end
        else
           [h]=plotFeasibleSetOfGSIP2D(problem,indicesFreePars,valuesFixedPars,lb,ub,nrPointsInDir);
        end
        
    case 3
        if nargin>6
            if strcmp(varargin{1},'contour')
                [h]=plotFeasibleSetOfGSIP3D2(problem,indicesFreePars,valuesFixedPars,lb,ub,nrPointsInDir);
            else
                warning('To get a picture with boundare give as a last input ''contour''');
            end
        else
           [h]=plotFeasibleSetOfGSIP3D(problem,indicesFreePars,valuesFixedPars,lb,ub,nrPointsInDir) ;
        end
    otherwise
        error('we can only plot problems with one, two or three free parameters');
end

if nargout==1
    varargout{1}=h;
end

end

