function [ ] = checkDesign( design,checkDeriv,checkSecDeriv)
%checkDesign(d,checkDeriv,checkSecDeriv) checks all functions of the
%Design d. If a output has the wrong size or an inconsistent value a
%warning is thrown. It may happen, that in a first run only a few warnings
%are thrown. If they are corrected more mistakes may be found. If
%checkDeriv is 1, derivatives are checked. if checkSecDeriv is 1, seccond
%derivatives are checked.

%Testing all get Functions
bgetDim=checkOneFunction(design,'getDim',{[1;1]});
bgetNrParams=checkOneFunction(design,'getNrParams',{[1;1]});
bgetNrNonlinDescrFuns= checkOneFunction(design,'getNrNonlinDescrFuns',{[1;1]});
bgetNrNonlinEqConstrsOnParams=checkOneFunction(design,'getNrNonlinEqConstrsOnParams',{[1;1]});
bgetNrNonlinIneqConstrsOnParams=checkOneFunction(design,'getNrNonlinIneqConstrsOnParams',{[1;1]});

bgetInitValsParams=0;
bgetLinEqsOnParams=0;
bgetLinIneqsOnParams=0;
bgetBoundsOnParams=0;
if bgetNrParams
    nrOfParams=design.getNrParams();
    bgetInitValsParams=checkOneFunction(design,'getInitValsParams',{[1;nrOfParams]});
    bgetLinEqsOnParams=checkOneFunction(design,'getLinEqsOnParams',{[inf,nrOfParams],[inf,1]});
     if bgetLinEqsOnParams
        [C,~]=design.getLinEqsOnParams;
        bgetLinEqsOnParams=checkOneFunction(design,'getLinEqsOnParams',{[size(C,1),nrOfParams],[size(C,1),1]});
    end
    bgetLinIneqsOnParams=checkOneFunction(design,'getLinIneqsOnParams',{[inf,nrOfParams],[inf;1]});
    if bgetLinIneqsOnParams
        [A,~]=design.getLinIneqsOnParams;
        bgetLinIneqsOnParams=checkOneFunction(design,'getLinIneqsOnParams',{[size(A,1),nrOfParams],[size(A,1),1]});
    end
    
    bgetBoundsOnParams=checkOneFunction(design,'getBoundsOnParams',{[1;nrOfParams],[1,nrOfParams]});
    
    
end
%Testing the set Function
bsetInitValsParams=0;   
if bgetInitValsParams
    initVals=design.getInitValsParams();
    bsetInitValsParams=checkOneFunction(design,'setInitValsParams',{},initVals);    
end
    
    
if bgetInitValsParams && bgetNrParams && bgetDim
    x=design.getInitValsParams;
    dim=design.getDim;
    bvol=checkOneFunction(design,'vol',{[1;1]},x);
    if bvol && checkDeriv
         bvol=checkOneFunctionDerivative(design,'gradVol',1,'vol',1,{[]},x,1);
         bvol=min(bvol,checkOneFunctionDerivative(design,'DxVol',0,'vol',1,{[]},x,1));  
    end
    if bvol &&checkSecDeriv
        bvol=checkOneFunctionSecDerivative(design,'DxDxVol','vol',1,{[]},x,1); 
    end
                   
    
    bgetInteriorPoint=checkOneFunction(design,'getInteriorPoint',{[1;dim]},x);
    
    bevalNonlinDescrFuns=0;
    
    bevalInvTrafoFun=0;
    bevalTrafoFun=0;
    if bgetInteriorPoint && bgetNrNonlinDescrFuns
        nrOfDesignConstrs=design.getNrNonlinDescrFuns;
        y=design.getInteriorPoint(x);
        bevalNonlinDescrFuns=checkOneFunction(design,'evalNonlinDescrFuns',{[1;nrOfDesignConstrs]},x,y);
        
        if bevalNonlinDescrFuns && checkDeriv
            min(bevalNonlinDescrFuns,checkOneFunctionDerivative(design,'DxNonlinDescrFuns',0,'evalNonlinDescrFuns',1,{[nrOfDesignConstrs]},x,1,y,1));
            min(bevalNonlinDescrFuns,checkOneFunctionDerivative(design,'DyNonlinDescrFuns',0,'evalNonlinDescrFuns',2,{[nrOfDesignConstrs]},x,1,y,1));
        end
        if bevalNonlinDescrFuns && checkSecDeriv
            min(bevalNonlinDescrFuns,checkOneFunctionSecDerivative(design,'DxDxNonlinDescrFuns','evalNonlinDescrFuns',1,{[nrOfDesignConstrs]},x,1,y,1));
            min(bevalNonlinDescrFuns,checkOneFunctionSecDerivative(design,'DyDyNonlinDescrFuns','evalNonlinDescrFuns',2,{[nrOfDesignConstrs]},x,1,y,1));
            min(bevalNonlinDescrFuns,checkOneFunctionMixedSecDerivative(design,'DxDyNonlinDescrFuns','evalNonlinDescrFuns',2,1,{[nrOfDesignConstrs]},x,1,y,1));
        end
        
        bevalInvTrafoFun=checkOneFunction(design,'evalInvTrafoFun',{[1;inf]},x,y);
        if bevalInvTrafoFun
            z=design.evalInvTrafoFun(x,y);
            bevalTrafoFun=checkOneFunction(design,'evalTrafoFun',{[1;dim]},x,z);
            
            if bevalTrafoFun && checkDeriv
                bevalTrafoFun=min(bevalTrafoFun,checkOneFunctionDerivative(design,'DxTrafoFun',0,'evalTrafoFun',1,{[dim]},x,1,z,1));
                bevalTrafoFun=min(bevalTrafoFun,checkOneFunctionDerivative(design,'DzTrafoFun',0,'evalTrafoFun',2,{[dim]},x,1,z,1));
            end
            if bevalTrafoFun && checkSecDeriv
                bevalTrafoFun=min(bevalTrafoFun,checkOneFunctionSecDerivative(design,'DxDxTrafoFun','evalTrafoFun',1,{[dim]},x,1,z,1));
                bevalTrafoFun=min(bevalTrafoFun,checkOneFunctionSecDerivative(design,'DzDzTrafoFun','evalTrafoFun',2,{[dim]},x,1,z,1));
                bevalTrafoFun=min(bevalTrafoFun,checkOneFunctionMixedSecDerivative(design,'DxDzTrafoFun','evalTrafoFun',2,1,{[dim]},x,1,z,1));
            end         
        end
             
    end
    
    bgetLinDescrFuns=checkOneFunction(design,'getLinDescrFuns',{[inf;dim],[inf,1]},x);
    if bgetLinDescrFuns
        [A]=design.getLinDescrFuns(x);
        nrLinIneqs=size(A,1);
        bgetLinDescrFuns=checkOneFunction(design,'getLinDescrFuns',{[nrLinIneqs,dim],[nrLinIneqs,1]},x); 
    end
    if bgetLinDescrFuns && checkDeriv
         [A]=design.getLinDescrFuns(x);
         nrLinIneqs=size(A,1);
         bgetLinDescrFuns=min(bgetLinDescrFuns,checkOneFunctionDerivative(design,'DxLinDescrFuns',0,'getLinDescrFuns',1,{[nrLinIneqs,dim],[nrLinIneqs,1]},x,1));
    end
    if bgetLinDescrFuns && checkSecDeriv
         [A]=design.getLinDescrFuns(x);
         nrLinIneqs=size(A,1);
         bgetLinDescrFuns=min(bgetLinDescrFuns,checkOneFunctionSecDerivative(design,'DxDxLinDescrFuns','getLinDescrFuns',1,{[nrLinIneqs,dim],[nrLinIneqs,1]},x,1));
    end
    
    bgetBoundingBox=checkOneFunction(design,'getBoundingBox',{[1;dim],[1,dim]},x);
    if bgetBoundingBox && checkDeriv
        bgetBoundingBox=min(bgetBoundingBox,checkOneFunctionDerivative(design,'DxBoundingBox',0,'getBoundingBox',1,{[dim],[dim]},x,1));
    end
    if bgetBoundingBox && checkSecDeriv
        bgetBoundingBox=min(bgetBoundingBox,checkOneFunctionSecDerivative(design,'DxDxBoundingBox','getBoundingBox',1,{[dim],[dim]},x,1));
    end
    

    if bgetNrNonlinEqConstrsOnParams && bgetNrNonlinIneqConstrsOnParams
        n1=design.getNrNonlinEqConstrsOnParams;
        n2=design.getNrNonlinIneqConstrsOnParams;
        %constraints on parameters
        bevalNonlinEqConstrsOnParams=checkOneFunction(design,'evalNonlinEqConstrsOnParams',{[1;n1]},x);
        
        if bevalNonlinEqConstrsOnParams && checkDeriv
            bevalNonlinEqConstrsOnParams=checkOneFunctionDerivative(design,'DxNonlinEqConstrsOnParams',0,'evalNonlinEqConstrsOnParams',1,{n1},x,1);
        end
        if bevalNonlinEqConstrsOnParams &&checkSecDeriv
            bevalNonlinEqConstrsOnParams=checkOneFunctionSecDerivative(design,'DxDxNonlinEqConstrsOnParams','evalNonlinEqConstrsOnParams',1,{n1},x,1);
        end
        
       
        bevalNonlinIneqConstrsOnParams=checkOneFunction(design,'evalNonlinIneqConstrsOnParams',{[1;n2]},x);
        if bevalNonlinIneqConstrsOnParams && checkDeriv
            bevalNonlinIneqConstrsOnParams=checkOneFunctionDerivative(design,'DxNonlinIneqConstrsOnParams',0,'evalNonlinIneqConstrsOnParams',1,{n2},x,1);
        end
        if bevalNonlinIneqConstrsOnParams &&checkSecDeriv
            bevalNonlinIneqConstrsOnParams=checkOneFunctionSecDerivative(design,'DxDxNonlinIneqConstrsOnParams','evalNonlinIneqConstrsOnParams',1,{n2},x,1);
        end
        

    end
    
allcorrect=min([bgetDim, bgetNrParams, bgetNrNonlinDescrFuns,...
bgetNrNonlinEqConstrsOnParams, bgetNrNonlinIneqConstrsOnParams, bgetInitValsParams, bgetLinEqsOnParams,...
bgetLinIneqsOnParams, bgetBoundsOnParams, bsetInitValsParams, bvol, bgetInteriorPoint,...
bevalNonlinDescrFuns, bevalInvTrafoFun, bevalTrafoFun, bgetLinDescrFuns, bgetBoundingBox,...
bevalNonlinEqConstrsOnParams, bevalNonlinIneqConstrsOnParams,]); 

if allcorrect
    disp('All methods passed the tests');
end        

end
    
    
    
        


   

end

