classdef ContainerC3 < Container
    %C=ContainerC3() creates an instance of the container C given by:
    %
    %       C={ y in R^2 with:
    %               1-y1^2-y2^2 <=0
    %               0.25-(y1-1.5)^2-y2^2<=0
    %               (y1-0.5)^2+y2^2-2.25 <=0
    %               -y2 <=0}
    %
    %   Bounding box: [-1,7]x[-1,1]
    
    properties (GetAccess=protected)
        dim=2;                              
        nrOfConstFuns = 4;                  
        BoundingBox=[-1,0;2,1.5]
    end

    
    methods 
        %container functions
        function g=evalDescrFuns(~,i,y) 
            m=size(y,1);
            o=ones(m,1);
            switch i
                case 1
                    g=1-y(:,1).^2-y(:,2).^2;
                case 2
                    g=1/4*o-(y(:,1)-3/2*o).^2-y(:,2).^2;
                case 3
                    g=(y(:,1)-1/2*o).^2+y(:,2).^2-9/4*o;
                case 4
                    g=-y(:,2);                   
             end

        end
        %derivative
        function Dy_g=DyDescrFuns(~,i,y)  
            m=size(y,1);
            o=ones(m,1);
            z=zeros(m,1);
            switch i
                case 1
                    Dy_g=[-2*y(:,1),-2*y(:,2)];
                case 2
                    Dy_g=[-2*(y(:,1)-3/2*o),-2*y(:,2)];
                case 3
                    Dy_g=[2*(y(:,1)-1/2*o),2*y(:,2)];
                case 4
                    Dy_g=[z,-o];                
            end
        end
    end
    
       methods
        function plotContainer(obj)
            t1 = linspace(pi,0,180);
            t2 = linspace(0,pi,180);
            t3 = linspace(0,pi,180);
            
            m=size(t1,2);
            o = ones(1,m);
            y1=zeros(1,3*m);
            y2=zeros(1,3*m);
            
            %3 circles
            y1(1:m) = 0.5*o+1.5*cos(t1);
            y2(1:m) = 1.5*sin(t1);
            
            y1(m+1:2*m) = 1.5*o+0.5*cos(t2);
            y2(m+1:2*m) = 0.5*sin(t2);
            
            y1(2*m+1:end) = cos(t3);
            y2(2*m+1:end) = sin(t3);
            
            ph = fill(y1, y2, [135/255, 206/255, 250/255]); % sky blue
            set(ph, 'EdgeColor', 'blue');
            hold on;
        end
    end 

    
    
end


