
classdef ContainerC6 < Container
    %C=ContainerC6() creates an instance of the container C given by:
    %
    %       C={ y in R^2 with:
    %               (y1-0.5)^2-y2-1 <=0
    %               y1^2+0.3y2^2-1<=0
    %
    %   Bounding box: [-0.5,1]x[-1,0]
   
    properties (GetAccess=protected)
        dim=2;                                
        nrOfConstFuns=2;                       
        BoundingBox=[-0.5,-1;1,0]

    end
    

    
    methods 
        %container functions
        function g=evalDescrFuns(~,i,y)   
            
            switch i
                case 1
                    g=(y(:,1)-0.5).^2-y(:,2)-1;
                case 2
                    g=y(:,1).^2+y(:,2);
            end

        end
    %derivative
        function Dy_g=DyDescrFuns(~,i,y)  
            m=size(y,1);
            
            switch i
                case 1
                    Dy_g=[2*(y(:,1)-0.5),-ones(m,1)];
                case 2
                    Dy_g=[2*y(:,1),ones(m,1)];                
            end
        end
    end
    
    methods
        function plotContainer(obj)
            X1 =linspace(0.25-sqrt(7)/4,0.25+sqrt(7)/4,100);
            X2 =linspace(0.25+sqrt(7)/4,0.25-sqrt(7)/4,100);
            Y = [(X1-0.5).^2-1,-(X2).^2];    
   
            ph = fill([X1,X2], Y, [135/255, 206/255, 250/255]); % sky blue
            set(ph, 'EdgeColor', 'blue');
            hold on;
        end
    end
    
    
end