
classdef Problem12< ImplicitGSIPProblem
    % p=Problem12 creates an instance of the GSIP problem:
    %
    %       min   x2
    %
    %       s.t.  -y^3+x2 <=0 for all y in Y(x)
    %             -1 <=x1 <= 1
    %             -1 <=x2 <= 1
    %             2x2-x1^2 <=-10^-6
    %
    %       where:
    %
    %       Y(x)={y in R with
    %              2x2-y^3-x1^2 <= 0
    %              -1 <= y <= 0}
    
    properties(GetAccess=protected)
        
        nrOfUppLevVars=2;                          
        boundsOnUppLevVars=[0,0;1,1];               
        nrOfSemiInfConstrs=1;                       
        initValsOfUppLevVars=[1,0];                
        
        linEqsOnUppLevVars=[];                     
        linIneqsOnUppLevVars=[];                   
        nrOfNonlinEqsUppLev=0;                
        nrOfNonlinIneqsUppLev=0;              
        
        
        nrOfLowLevVars=3;                          
        nrOfLowLevNonlinConstrs=1;
        
        con2LowLev=1;
        nrOfIndexset=1;        

        
    end

    methods 
        %objective function
        function f=evalObjFun(obj,x)                   
            f=0.5*x(1)^4+2*x(1)*x(2)-2*x(1)^2;
        end
        function grad_f=gradObjFun(obj,x)                    
            grad_f=[2*x(1)^3+2*x(2)-4*x(1);2*x(1)];
        end
        
        %upper level
        function g=evalSemiInfConstrs(obj,i,x,y)                                                    
            g=y(:,1).^2+y(:,2).^2-x(1)+x(1)^2-x(2);            
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x,y)     
            m=size(y,1);
            o=ones(m,1);
            Dx_g=[(-1+2*x(1))*o,-o];                
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)
            m=size(y,1);
            Dy_g=[2*y(:,1),2*y(:,2),zeros(m,1)];                
        end
        
        %lower level
        function u=evalLowLevNonlinConstrs(obj,~,x,y)         
            u=[y(1)^2+y(2)^2+y(3)^2-x(1)];                        
        end
        function Dy_u=DyLowLevNonlinConstrs(obj,~,x,y)              
            Dy_u=[2*y(1),2*y(2),2*y(3)];                        
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,~,x,y)              
            Dx_u=[-1,0];
        end
            
        function y=getInitValsLowLevVars(obj,~,x)           
            y=1/2*[x(1),x(1),x(1)];                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)
            Dx_y=1/2*[ones(3,1),zeros(3,1)];
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,x)    
             lb=[0,0,0];
             ub=[1,1,1];
        end
                        
        
        %Transformation
        function y=evalTrafoFun(~,~,x,z) 
            y=x(1).*z;            
        end
        
        function z=evalInvTrafoFun(~,~,x,y) 
            z=y./x(1);                  
        end
        %derivative
        function Dx_y=DxTrafoFun(~,~,x,z) 
            n=size(z);
            Dx_y=zeros(3,2,size(z,1));
            Dx_y(:,1,:)=z';         
        end
        
    end
    
end
