classdef  Problem5 < ImplicitGSIPProblem
    % p=Problem5() creates an instance of the GSIP problem:
    %
    %       min   x1^2+x2^2
    %
    %       s.t.  -(y1-x1)^2-(y2-x2)^2+1 <=0 for all y in Y(x) 
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               -y1 <= -x1
    %               -y2 <= 0}
    
    properties  (GetAccess=protected)
                
        nrOfUppLevVars =2;
        boundsOnUppLevVars =[-inf,-inf; inf, inf];
        nrOfSemiInfConstrs =1;
        initValsOfUppLevVars= [2 -1];
        
        linEqsOnUppLevVars =[];
        linIneqsOnUppLevVars =[];
        nrOfNonlinIneqsUppLev=0;
        nrOfNonlinEqsUppLev=0;
        
        nrOfLowLevVars =2;
        nrOfLowLevNonlinConstrs =0;
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end
    
    
    methods
        %objective function
        function f=evalObjFun(~,x)                       
            f=x(1)^2+x(2)^2;
        end
        function grad_f=gradObjFun(~,x)
            grad_f=[2*x(1); 2*x(2)];
        end
        
        %upper level
        function g=evalSemiInfConstrs(~,~,x,y)      
           g=-(y(:,1)-x(1)).^2-(y(:,2)-x(2)).^2+1;
           
        end
        function Dx_g=DxSemiInfConstrs(~,~,x,y)
            D1=2*(y(:,1)-x(1));
            D2=2*(y(:,2)-x(2));

            Dx_g=[D1, D2];
        end
        function Dy_g=DySemiInfConstrs(~,~,x,y)

            D3=-2*(y(:,1)-x(1));
            D4=-2*(y(:,2)-x(2));
            Dy_g=[D3, D4];
        end
                
        %lower level
      
        function y=getInitValsLowLevVars(~,~,x)        
        y=[x(1)+1,1];
        end
        function Dx_y=DxStartingPointLowLev(~,~,~)
            Dx_y=[1,0;0,0];
        end
        
                %Linear inequalities
        function [A,b]=getLinIneqsOnLowLevVars(obj,k,x)
            A=[-1,0;0,-1];
            b=[-x(1);0];
        end

        
        %transformation
        function y=evalTrafoFun(~,~,x,z) 
            y=[x(1)+z(:,1),z(:,2)];
            
        end
        
        function z=evalInvTrafoFun(~,~,x,y)  
            z=[y(:,1)-x(1),y(:,2)];
        end
        
        function Dx_y=DxTrafoFun(~,~,~,y)    
            m=size(y,1);
            Dx_y=zeros(2,2,m);
            Dx_y(1,1,:)=ones(1,m);
        end
    end
    
end

